// Bandit.java                                  MM 2010

/**
 * Der einarmige Bandit:
 * Simulation eines Spielautomaten als Konto.
 */
public class Bandit extends Konto
{
/* ------------------------------------------------- */
                                            // Attribut
/**
 * Wuerfelsatz, meist drei Stueck.
 */
  private Wuerfel[] wuerfel;

/* ------------------------------------------------- */
                                       // Konstruktoren
/**
 * Konstruktor, legt Konto an, erzeugt drei Wuerfel.
 * @param min minimale Augenzahl der Wuerfel
 * @param max maximale Augenzahl der Wuerfel
 * @param euro Startkapital im Automaten
 */
  public Bandit( int min, int max, int euro)
  {
// Kapital
    super( euro);
    
// Wuerfelsatz
    wuerfel = new Wuerfel[ 3];
    for( int i = 0; i < 3; i++)
      wuerfel[ i] = new Wuerfel( min, max);
  }

/**
 * Konstruktor, Standardautomat,
 * legt Konto an, erzeugt drei Wuerfeln 0 .. 9.
 * @param euro Startkapital im Automaten
 */
  public Bandit( int euro)
  {
    this( 0, 9, euro);
  }

/* ------------------------------------------------- */
                                    // toString-Methode
/**
 * Darstellen des Automatenzustandes mit aktuellem
 * Kapital und aktueller Zahlenanzeige als String.
 * @return Automatenzustand
 */
  public String toString()
  {
    String str = super.toString();             // Haben

    str += "\tZahlen ";                       // Zahlen
    for( int i = 0; i < wuerfel.length; i++)
      str += wuerfel[ i].getAugen() + " ";

    return str;
  }

/* ------------------------------------------------- */
                                         // get-Methode
/**
 * Liest aktuelle Zahlen des Automaten.
 * @return aktuelle Zahlen
 */
  public int[] getZahlen()
  {
    int[] zahlen = new int[ wuerfel.length];
    for( int i = 0; i < wuerfel.length; i++)
      zahlen[ i] = wuerfel[ i].getAugen();

    return zahlen;
  }

/* ------------------------------------------------- */
                                     // service-Methode
/**
 * Simulation der Funktionsweise des Standardautomaten:
 * Einwerfen des Spieleinsatzes, Ziehen des Armes,
 * Gewinnermittlung und Gewinnausschuettung.
 * @param euro Spieleinsatz
 * @return Gewinn
 */
  public int ziehen( int euro)
  {
// kein Einsatz
    if( euro <= 0) return 0;

// Spieleinsatz gutschreiben
    verbuchen( euro);

// Ziehen des Armes
    int[] zahlen = wuerfeln();

// Gewinnermittlung
    int F = 1;
    if( zahlen[ 0] == zahlen[ 1]) F++;
    if( zahlen[ 0] == zahlen[ 2]) F++;
    if( zahlen[ 1] == zahlen[ 2]) F++;

    int gewinn = 0;

    if( F != 1)
    {
      gewinn = F * euro;            // Gewinnberechnung

      int haben = getHaben();      // ueberprueft Haben
      if( gewinn > haben) gewinn = haben;

      verbuchen( -gewinn);       // Gewinnausschuettung
    }

    return gewinn;
  }

/**
 * Wuerfeln mit alle Wuerfeln.
 * @return gewuerfelte Zahlenfolge
 */
  private int[] wuerfeln()
  {
    int[] zahlen = new int[ wuerfel.length];
    for( int i = 0; i < wuerfel.length; i++)
      zahlen[ i] = wuerfel[ i].wuerfeln();

    return zahlen;
  }

/* ------------------------------------------------- */
                                        // Testprogramm
/**
 * Testet die Klasse Bandit,
 * erzeugt Standardautomat mit Anfangskapital,
 * legt Einsatz fest und zieht hoechstens 5 x.
 */
  public static void main( String[] args)
  {
// Automat einrichten (200 Euro Startkapital)
    System.out.println();
    Bandit bandit = new Bandit( 200);
    System.out.println
    ( "Automat (200 Euro Startkapital):\n" + bandit);

// Einsatz (150 Euro)
    System.out.println();
    int einsatz = 150;
    System.out.println( "Einsatz: " + einsatz);

// 5 * Armziehen bei unverndertem Einsatz
    int runden = 5;
    do
    {
      runden--;
      bandit.ziehen( einsatz);
      System.out.println( bandit);
    } while( runden > 0 && !bandit.istPleite());

// Beenden
    System.out.println();
    System.out.println( "Test beendet!");
  }
}
